"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const phpunit_1 = require("./phpunit");
const filesystem_1 = require("./filesystem");
const helpers_1 = require("./helpers");
const textline_range_1 = require("./phpunit/textline-range");
class Runner {
    constructor(phpUnit = new phpunit_1.PhpUnit(), collect = new phpunit_1.Collection(), ast = new phpunit_1.Ast(), files = new filesystem_1.Filesystem(), textlineRange = new textline_range_1.TextlineRange()) {
        this.phpUnit = phpUnit;
        this.collect = collect;
        this.ast = ast;
        this.files = files;
        this.textlineRange = textlineRange;
        this.lastCommand = undefined;
    }
    setBinary(binary) {
        return helpers_1.tap(this, () => {
            this.phpUnit.setBinary(binary);
        });
    }
    setDefault(args) {
        return helpers_1.tap(this, () => {
            this.phpUnit.setDefault(args);
        });
    }
    run(connection, uri, path, params = []) {
        return __awaiter(this, void 0, void 0, function* () {
            this.phpUnit.onRunning((command) => {
                this.lastCommand = {
                    path,
                    params,
                };
                connection.sendNotification('running', command);
            });
            this.phpUnit.onDone((output, tests) => {
                this.collect.put(tests);
                connection.console.log(output);
                connection.sendNotification('done', this.collect.toObject());
                this.sendDiagnostics(connection).sendNotification(connection, uri);
            });
            return helpers_1.tap(this, () => __awaiter(this, void 0, void 0, function* () { return yield this.phpUnit.run(path, params); }));
        });
    }
    runNearest(connection, uri, path, params = []) {
        return __awaiter(this, void 0, void 0, function* () {
            const filter = this.getMethodFilter(yield this.textlineRange.findMethod(path, parseInt(params[0], 10)));
            return helpers_1.tap(yield this.run(connection, uri, path, filter), () => this.textlineRange.clear());
        });
    }
    runLast(connection, uri, path, params = []) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.lastCommand) {
                path = this.lastCommand.path;
                params = this.lastCommand.params;
            }
            return yield this.run(connection, uri, path, params);
        });
    }
    getTestNodes(code, uri) {
        return this.ast.parse(code, this.files.uri(uri));
    }
    sendDiagnostics(connection) {
        return helpers_1.tap(this, () => {
            this.collect.getDiagnoics().forEach((diagnostics, uri) => {
                connection.sendDiagnostics({
                    uri,
                    diagnostics,
                });
            });
        });
    }
    sendNotification(connection, pathOrUri) {
        return helpers_1.tap(this, () => {
            const uri = this.files.uri(pathOrUri);
            connection.sendNotification('assertions', {
                uri: uri,
                assertions: this.collect.getAssertions().get(uri) || [],
            });
        });
    }
    getCodeLens(textDocument) {
        const uri = textDocument.uri;
        const testNodes = this.getTestNodes(textDocument.getText(), uri);
        return testNodes
            .concat(this.collect.getTestNodes(uri).filter((testNode) => {
            for (const node of testNodes) {
                if (node.uri === testNode.uri && testNode.range === testNode.range) {
                    return false;
                }
            }
            return true;
        }))
            .map((node) => this.transformTestNodeToCodeLen(node, uri));
    }
    transformTestNodeToCodeLen(node, uri) {
        return {
            range: node.range,
            command: helpers_1.when(node.class === node.name, () => {
                return {
                    title: 'Run Test',
                    command: 'phpunit.test.file',
                    arguments: [uri, this.files.normalizePath(node.uri), []],
                };
            }, () => {
                return {
                    title: 'Run Test',
                    command: 'phpunit.test',
                    arguments: [uri, this.files.normalizePath(node.uri), this.getMethodFilter(node.name)],
                };
            }),
            data: {
                textDocument: {
                    uri: uri,
                },
            },
        };
    }
    getMethodFilter(method) {
        return method ? ['--filter', `^.*::${method}$`] : [];
    }
}
exports.Runner = Runner;
//# sourceMappingURL=runner.js.map